#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
a Monitor for Memcachedb

Overview
========

See U{the Memcachedb homepage<http://memcachedb.googlecode.com>} for more about memcachedb.

Usage summary
=============

Command::

  python mdbtop.py <config file>

Monitor config file example::

  # here a bunch of servers you wanna monitor
  [Server]  
  server1 = 127.0.0.1:21201
  server2 = 127.0.0.1:21202
  [View]    
  interval = 2  # display interval for seconds

"""

import sys
import time
import signal
import ConfigParser

import memcache

__author__    = "Steve Chu <stvchu@gmail.com>"
__version__ = "0.0.1"
__copyright__ = "Copyright (C) 2008 Steve Chu"
__license__   = "Python"

class Monitor:
  def __init__(self, cfg_file):
    self.mdb_servers = []
    self.mdb_clients = []
    self.cfg = ConfigParser.ConfigParser()
    self.cfg.readfp(open(cfg_file))
    self.interval = int(self.cfg.get('View', 'interval'))
    for (key,value) in self.cfg.items('Server'):
      self.mdb_servers.append([value])
    for mdb_server in self.mdb_servers:
      self.mdb_clients.append(memcache.Client(mdb_server, debug = 0))

  def top(self):
    print 
    print 'mdbtop.py %s, %s' % (__version__, __author__)
    print 
    while True:
      for mdb_client in self.mdb_clients:
        # deamon stats
        stats = []
        stats = mdb_client.get_stats()
        if stats == []:
          continue
        print "=============================================================================="
        print "[%s] up: %s, threads: %s, ver: %s" % (stats[0][0], stats[0][1]['uptime'], stats[0][1]['threads'], stats[0][1]['version'],)
        print "------------------------------------------------------------------------------"
        print "%-11s  %10s  %10s  %13s  %10s  %10s" % ('ibuffer_size', 'curr_conns', 'bytes_read', 'bytes_written', 'cmd_get', 'cmd_set' )
        print "%-11s  %10s  %10s  %13s  %10s  %10s" % (stats[0][1]['ibuffer_size'], stats[0][1]['curr_connections'], stats[0][1]['bytes_read'], stats[0][1]['bytes_written'], stats[0][1]['cmd_get'], stats[0][1]['cmd_set'] )
        print 
        # bdb stats
        stats = []
        stats = mdb_client.get_stats('bdb')
        if stats == []:
          continue
        if stats[0][1]['txn_nosync'] == '1':
          txn_nosync = 'Yes'
        elif stats[0][1]['txn_nosync'] == '0':
          txn_nosync = 'No'
        else:
          txn_nosync = 'N/A'

        print "%-10s  %12s  %10s  %12s  %12s" % ('cache_size', 'txn_lg_bsize', 'txn_nosync', 'dldetect_val', 'chkpoint_val' )
        print "%-10s  %12s  %10s  %12s  %12s" % (stats[0][1]['cache_size'], stats[0][1]['txn_lg_bsize'], txn_nosync, stats[0][1]['dldetect_val'], stats[0][1]['chkpoint_val'] )
        print 
        # rep stats
        stats = []
        stats = mdb_client.get_stats('rep')
        if stats == [] or stats[0][1] == {}:
          continue

        if stats[0][1]['rep_is_master'] == 'REP_TRUE':
          rep_is_master = 'Yes'
        elif stats[0][1]['rep_is_master'] == 'REP_FALSE':
          rep_is_master = 'No'
        else:
          rep_is_master = 'N/A'

        print "%-13s  %21s  %21s  %12s" % ('rep_ismaster', 'rep_hp', 'rep_whoismaster', 'rep_priority')
        print "%-13s  %21s  %21s  %12s" % (rep_is_master, stats[0][1]['rep_localhp'], stats[0][1]['rep_whoismaster'], stats[0][1]['rep_priority'])
        print 

      if self.interval == 0:
        break
      else:
        time.sleep(self.interval)

  def close(self):
    for mdb_client in self.mdb_clients:
      mdb_client.disconnect_all()

def handler(signum, frame):
  sys.exit(0)

if __name__ == "__main__":
  if len(sys.argv) != 2:
    print '\nUsage:'
    print 'python mdbtop.py <config file>'
    sys.exit(1)
  cfg_file = sys.argv[1]

  signal.signal(signal.SIGTERM, handler)
  signal.signal(signal.SIGQUIT, handler)
  signal.signal(signal.SIGINT, handler)
  mon = Monitor(cfg_file)
  mon.top()
  mon.close()
